// mozileDetection.js

/* ***** BEGIN LICENSE BLOCK *****
 * Licensed under Version: MPL 1.1/GPL 2.0/LGPL 2.1
 * Full Terms at http://mozile.mozdev.org/license.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Playsophy code.
 *
 * The Initial Developer of the Original Code is Playsophy
 * Portions created by the Initial Developer are Copyright (C) 2002-2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *		Karl Guertin <grayrest@grayrest.com>
 *		James A. Overton <james@overton.ca>
 *		Christian Stocker (Bitflux)
 *		Paul Everitt
 *		David Palm
 *
 * ***** END LICENSE BLOCK ***** */

/*******

Mozile Detection Javascript
Mozile version 0.6.16

Created by James A. Overton
Created on 2004-03-24

Last revised by James A. Overton
Last revised on 2004-11-16

HISTORY:
	Some code in this file was first found in mozilehtmltb.js, while the 
majority was cut out of mozileTB.js and separated into different files.


DESCRIPTION:
	The code here is responsible for detecting when Mozile should be active.
It gets called a lot, so mozileDetect() has to be very fast.

*********/

function mozileGetCurrentTarget() {
	return mozileTarget;
}

/*
function mozileIsContentEditable(nodeToCheck) {
		while(nodeToCheck)
		{
			// find a non element node - return: probably DOCUMENT_NODE reached!
			if(nodeToCheck.nodeType != Node.ELEMENT_NODE)
				return 0;
			// if has a content editable attribute then sync it with mozusermodify
			if(nodeToCheck.getAttribute("contentEditable"))
			{
				if(nodeToCheck.getAttribute("contentEditable") == "true")
					return 1;
				return 0;
			}
			nodeToCheck = nodeToCheck.parentNode;
		}
		return 0;
}
*/

// Check all element nodes in the Document element for the contentEditable attribute.
// When found, add Mozile (in-line) style attributes to the element node.
// Only works if the Document element is <html> or <HTML>
//
function mozileConvertCE() {
	var f = ["mozileDetection.js","mozileConvertCE"];
	var target, count, l, i;
	try {
		target = document.commandDispatcher.focusedWindow;
		if ( target.toString() == "[object ChromeWindow]" ) {
			return null;
		}
		count = 0;
		if ( target.document.documentElement.tagName.toLowerCase() == "html" ) {
			l = target.document.documentElement.getElementsByTagName("*");
			for( i=0; i<l.length; i++ ) {
				if ( l[i].nodeType == Node.ELEMENT_NODE && l[i].hasAttributes() ) {
					if ( l[i].hasAttribute( "contentEditable" ) ) {
						if ( l[i].getAttribute( "contentEditable" ).toLowerCase() == "true" ) {
							l[i].style.MozUserModify = "read-write";
							l[i].style.MozUserInput = "enabled";
							l[i].style.MozUserSelect = "normal"; //Is this needed?
							++count;
						}
					}
				}
			}
			return count + " contentEditable tags made editable.";
		}
		else {
			return "The window does not contain an x/html document.";
		}
	}
	catch (e) {
		mozileDebug(f,1,"Exception: "+e);
		return null;
	}
}

// Make any html document editable.
// Adds the Mozile style tags to the first <body> element.
//
function mozileMakeEditable() {
	var f = ["mozileDetection.js","mozileMakeEditable"];
	var target, l;
	try {
		target = document.commandDispatcher.focusedWindow;
		if ( target.toString() == "[object ChromeWindow]" ) {
			return null;
		}
		if ( target.document.documentElement.tagName.toLowerCase() == "html" ) {
			l = target.document.documentElement.getElementsByTagName("body");
			if ( l.length > 0 ) {
				l[0].style.MozUserModify = "read-write";
				l[0].style.MozUserInput = "enabled";
				l[0].style.MozUserSelect = "normal";
				return "The Document is now editable.";
			} else {
				return "The Document does not contain an html <body> tag. It has not been made editable.";
			}
		} else {
				// A strange hack where I add a rule to an existing style-sheet. If non exists I add a new stylesheet
			l = target.document.documentElement;
			var ss = target.document.styleSheets
			mozileDebug(f,1,"CSS: "+ss +ss.length+ss[0]+l.localName);
				// if there are no stylesheets, create a processing instruction and insert it.
			if(ss.length==0){
				var pi = target.document.createProcessingInstruction("xml-stylesheet",'href="chrome://mozile/skin/mozile.css" type="text/css"');
				target.document.insertBefore(pi,l);
				ss = target.document.styleSheets;
				mozileDebug(f,1,"CSS Again: "+ss +ss.length+ss[0]+pi);
			}
				// now add a special new rule at the end of the first stylesheet
			ss[0].insertRule(l.localName + " { -moz-user-modify: read-write; -moz-user-input: enabled; -moz-user-select: normal; }", ss[0].cssRules.length);
			return "The Document is now editable.";
		
		//	return "The document does not have \"html\" as the root element. It has not been made editable.";
		}
	}
	catch (e) {
		mozileDebug(f,1,"Exception: "+e);
		return null;
	}
}


// Mozile Detect
// Check for the mozileChromeLoaded flag.
// Check to see if the user has selected an editable area.
// If so, set the global mozileTarget and mozileSelection variables.
// If not, return 0, and don't do anything else.

function mozileNoDetect(){
// Commented out because minor problems updating toolbar when using multiple windows and loading new documents.
// if(arguments.length==0 || arguments[0]==true){
	mozileClearCommandMenu();	mozileClearButtons();	mozile.lastContextNode=null;
// }
 return 0;
}

function mozileDetect() {
	try {
		// get the focused window
		var mozileCheckTarget = document.commandDispatcher.focusedWindow;
		//Ignore if in browser.
		if ( mozileCheckTarget.toString() == "[object ChromeWindow]" ) {
			return 0;	}
		//Don't think this second test is needed.
//		if(mozileCheckTarget.document.documentElement==document.documentElement) {
//			return 0;	} 
	} catch (e) {
		return mozileNoDetect();
	}
	try{
		var f = ["mozileDetection.js","mozileDetect"];
		//get the selection of the focused window
		var mozileCheckSelection = mozileCheckTarget.getSelection();

		// if there's no node, Mozile won't be able to edit
		try {
  		if ( mozileCheckSelection.anchorNode == null ) {
				throw "anchorNode==null"; }
  		if ( mozileCheckSelection.anchorNode.parentNode == null ) {
				throw "parentNode==null"; }
			var currentContextNode = mozileGetContext(mozileCheckSelection);
			if ( currentContextNode == null ) {
				throw "null return from mozileGetContext"; }
		} catch(e) {
			mozileDebug(f,3,"No node to edit: "+e);
			return mozileNoDetect(("mozileChromeLoaded" in mozileCheckTarget));
		}
		mozileDebug(f,3,"Detecting node: "+currentContextNode.tagName);

			// don't edit these nodes!			
		var ignoreList = new Array("input","textarea");
		for(var i=0; i<ignoreList.length; i++) {
			if(currentContextNode.tagName.toLowerCase()==ignoreList[i]) {
				return mozileNoDetect(("mozileChromeLoaded" in mozileCheckTarget));
			}
		}

/*
			//[MAX - this shouldn't be needed if changes to undo work correctly...
			// for x/html documents you would need to detect the <body> element as well...]
			// 
			// undo/redo can change the selection to the documentElement, so ignore a sudden change
		if( currentContextNode == mozileCheckTarget.document.documentElement ) {
			mozileDebug(f,0,"The current node is the document element");
			return 1;
		}*/

			// is it the same (editable) node?
			//This is a short circuit to save executing the code below...
		try{
			if(mozile.lastContextNode === currentContextNode) {
				mozileDebug(f,3,"We're still inside the same editable node");
				return 1;
			}
		} catch(e) { 
			throw e;
		}

		// Check if the selection is editable. Check CSS first, it's fast.
		var CSSenabled = mozileCheckTarget.document.defaultView.getComputedStyle(currentContextNode, null).MozUserModify;
		mozileDebug(f,3,"CSS Enabled? "+CSSenabled);
		if(CSSenabled!="read-write") {
			return mozileNoDetect(("mozileChromeLoaded" in mozileCheckTarget));
		}

			// if mozileChromeLoaded is false, inject the code
		var mozileInjected=0;
		if( !("mozileChromeLoaded" in mozileCheckTarget) || mozileCheckTarget.mozileChromeLoaded != "true") {
			mozileInject(mozileCheckTarget);
			mozileInjected=1;
		}

		//Set global variables	
		mozileTarget = mozileCheckTarget;
		mozileSelection = mozileCheckSelection;
		mozile.lastContextNode = currentContextNode;
		try {
			mozileUpdateCommandMenu();
			mozileUpdateButtons();
			if (mozileInjected) {
				mozileUpdateUndoButtons();
			}
		}
		catch(e) {
			mozileDebug(f,1,"Exception while updating: "+e);
		}
		mozileDebug(f,3,"Detect done.");
		return 1;	// done
	}
	catch(e) { 
		mozileDebug(f,0,"Detect Died: "+e);
		return mozileNoDetect();
	}
}
