// mozileToolbar.js

/* ***** BEGIN LICENSE BLOCK *****
 * Licensed under Version: MPL 1.1/GPL 2.0/LGPL 2.1
 * Full Terms at http://mozile.mozdev.org/license.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Playsophy code.
 *
 * The Initial Developer of the Original Code is Playsophy
 * Portions created by the Initial Developer are Copyright (C) 2002-2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *		Karl Guertin <grayrest@grayrest.com>
 *		James A. Overton <james@overton.ca>
 *		Christian Stocker (Bitflux)
 *		Paul Everitt
 *		David Palm
 *
 * ***** END LICENSE BLOCK ***** */

/*******

Mozile Toolbar Javascript
Mozile version 0.6.16

Created by James A. Overton
Created on 2004-03-24

Last revised by James A. Overton
Last revised on 2004-11-16

HISTORY:
	Some code in this file was first found in mozilehtmltb.js, while the 
majority was cut out of mozileTB.js and separated into different files.


DESCRIPTION:
	The code here deals with all of the active toolbar stuff, like removing
and replacing buttons and menus.

*********/


/******** Check For Active Styles ********/
function mozileCheckStyle(styleName, styleValue) {
	var active="false";

	var textContainer = mozileGetContext(mozileSelection);
	if(textContainer) { 
		var result=mozileTarget.document.defaultView.getComputedStyle(textContainer, null).getPropertyValue(styleName);
		if(result==styleValue) active="true";
	}

	return active;
}

function mozileCheckActive(carr) {
	var active="false";
	var type = carr[6];
	var args = carr[7];

	switch(type) {
		case "toggleStyle":
			active=mozileCheckStyle(args[0],args[1]);
		break;

		case "toggleAttribute":
			attrVal=mozileGetContext(mozileSelection).getAttribute(args[0]);
			if(args[1]==attrVal) active="true";
		break;
	}
	//alert(commandArray[0]+active);
	return active;
}


/******** Command Menu ********/

function mozileUpdateCommandMenu() {
	var f = ["mozileToolbar.js","mozileUpdateCommandMenu"];
	mozileDebug(f,3,"Updating Command Menu...");

	try {
		var contextTag = mozileGetContext(mozileSelection).tagName.toLowerCase();
		var tagIndex = findByName(contextTag, mesTags);
		if(tagIndex<0) {
			//alert("Context "+ contextTag +" is undefined!"); 
			mozileClearCommandMenu();	// clears the menu
			return; 
		}
		
		if(!mozileToolbar || mozileToolbar=="") {
			mozileToolbar = document.getElementById("mozile-toolbar");
		}
	
		mozileMakeCommandMenu(mesTags[tagIndex][3]);
	} catch(e) { alert("Update Command Menu Error: "+e); }
}


function mozileClearCommandMenu() {
	if(mozileCommandMenu=="") {
		mozileCommandMenu = document.getElementById("mozile-command-menu");
	}
	mozileRemoveChildren(mozileCommandMenu);
	return;
}

function mozileMakeCommandMenu(commandsArray) {

	//alert("CN:"+commandsArray);
	if(mozileCommandMenu=="") {
		mozileCommandMenu = document.getElementById("mozile-command-menu");
	}
	mozileClearCommandMenu();
	var menupop = mozileMakeMenu(commandsArray);
	mozileCommandMenu.appendChild(menupop);
}



/******** General Menus ********/

function mozileMakeMenu(commandsArray) {
	var menupopup = document.createElement('menupopup');
	for(var i=0; i<commandsArray.length; i++) {
		var menuitem = mozileMakeMenuitem(commandsArray[i]);
		if(menuitem!=0) { menupopup.appendChild(menuitem); }
	}
	return menupopup;
}


function mozileMakeMenuitem(commandArray) {
		// check to see if this command is in the mesCommands array
	var commandIndex = findByName(commandArray[0], mesCommands);
		// if not, see if it's a commandGroup
	if(commandIndex<0) {
		if(commandArray.length!=7) {
			alert("Can't find this command!");
			return 0;
		}	
		
			// if it has the right shape, make a new 
		var garr = new Array();
		garr = commandArray;
		var menu = document.createElement('menu');
		menu.setAttribute('label',garr[1]);
		menu.setAttribute('accesskey',garr[4]);
		menu.setAttribute('tooltiptext',garr[2]);
		var newMenupopup = mozileMakeMenu(garr[6]);		// call this function recursively
		menu.appendChild(newMenupopup);
		return menu;
	}
	
	var carr = new Array();
	carr = mesCommands[commandIndex];
	
	var menuitem = document.createElement('menuitem');
	menuitem.setAttribute('label',carr[1]);
	menuitem.setAttribute('accesskey',commandArray[1]);
	menuitem.setAttribute('tooltiptext',carr[2]);
	menuitem.setAttribute('oncommand','mozileFormat("'+ carr[0] +'")' ); // use the command name
		// if the command is currently active, then put a check next to the menuitem
	if(mozileCheckActive(carr)=="true") {
		menuitem.setAttribute("type","checkbox");
		menuitem.setAttribute("checked","true");
	}
	
	return menuitem;

}


/******** Toolbar Context Buttons ********/

function mozileClearButtons() {
	var f = ["mozileToolbar.js","mozileClearButtons"];
	mozileDebug(f,3,"Clearing buttons...");
	var separator = 0;
	var count = 0;
	
	if(!mozileArrowbox || mozileArrowbox=="") {
		var mozileArrowbox = document.getElementById('mozile-arrowbox');
	}
	
	mozileArrowbox.scrollByIndex(-100);
	mozileRemoveChildren(mozileArrowbox);
	
	/*
	var children = mozileArrowbox.childNodes;
	for(var i=0; i<children.length; i++) {
		if(children[i].id=="mozile-context-buttons-start") {
			separator = 1;		
		} else {
			if(separator) { 
				mozileToolbar.removeChild(children[i]);
				count++;
			}
		}
	}
	mozileDebug(f,3, count+" buttons removed");
	*/
}




function mozileUpdateButtons() {
	var f = ["mozileToolbar.js","mozileUpdateButtons"];
	mozileDebug(f,3,"Updating buttons...");

	var contextTag = mozileGetContext(mozileSelection).localName.toLowerCase();
	var tagIndex = findByName(contextTag, mesTags);
	
	if(tagIndex<0) {
		mozileDebug(f,2,"Context undefined!"); 
		mozileClearButtons();
		return; 
	}
	
	if(!mozileToolbar || mozileToolbar=="") {
		mozileToolbar = document.getElementById("mozile-toolbar");
	}
	
	mozileClearButtons();
	mozileMakeButtons(mesTags[tagIndex][3]);
}



/******** Undo Redo Buttons ********/

function mozileUpdateUndoButtons() {
	var f = ["mozileToolbar.js","mozileUpdateUndoButtons"];
	mozileDebug(f,3,"Updating Undo Buttons...");

	var undo0 = "chrome://mozile/skin/buttons/undo-disabled.png";
	var undo1 = "chrome://mozile/skin/buttons/undo.png";
	var redo0 = "chrome://mozile/skin/buttons/redo-disabled.png";
	var redo1 = "chrome://mozile/skin/buttons/redo.png";

/*	if(undo=="") {
		undo=document.getElementById("mozile-undo")
	}
	if(redo=="") {
		redo=document.getElementById("mozile-redo")
	}*/
	var u=document.getElementById("mozile-undo")
	var r=document.getElementById("mozile-redo")
	
	try {
		if ( "undoStack" in mozileTarget ) {
			var undoStack = mozileTarget.undoStack.length - 1;
		} else {
  		u.setAttribute("image",undo0);
  		r.setAttribute("image",redo0);
  		return;
		}
	} catch(e) { 
		u.setAttribute("image",undo0);
		r.setAttribute("image",redo0);
		return;
	}
	
	var undoCounter = mozileTarget.undoCounter;
	
		// deal with undo
	if(undoCounter==0) {
		u.setAttribute("image",undo0);
	} else {
		u.setAttribute("image",undo1);
	}
		// deal with redo
	if(undoStack<=undoCounter) {
		r.setAttribute("image",redo0);
	} else {
		r.setAttribute("image",redo1);
	}
}




/******** General Buttons ********/
function mozileMakeButtons(commandsArray) {

	var button;
	for(var i=0; i<commandsArray.length; i++) {
			// only make a button if the button attribute is "true"
			// normal command case
		var mozileArrowbox = document.getElementById('mozile-arrowbox');
			
		var commandArray=commandsArray[i];
		if(commandArray.length==3 && commandArray[2]=="true") { 
			button = mozileMakeButton(commandArray);
			if(button!=0) { mozileArrowbox.appendChild(button); }
		}
			// command group case
		if(commandArray.length==7 && commandArray[5]=="true") { 
			button = mozileMakeButton(commandArray);
			if(button!=0) { mozileArrowbox.appendChild(button); }
		}
	}
}


function mozileMakeButton(commandArray) {
		// check to see if this command is in the mesCommands array
	var button, image;
	var commandIndex = findByName(commandArray[0], mesCommands);
		// if not, see if it's a commandGroup
	if(commandIndex<0) {
		if(commandArray.length!=7) {
			alert("Can't find this command!");
			return 0;
		}	
		
			// if it has the right shape, make a menu button
		var garr = new Array();
		garr = commandArray;
		
		image = mesImages[0]+garr[3]+mesImages[2];
		
		button = document.createElement('toolbarbutton');
		button.id="mozile-"+garr[0]+"-menu";
		button.setAttribute('class','mozile-toolbar-menu');
		button.setAttribute('label',garr[1]);
		button.setAttribute('image',image);
		button.setAttribute('tooltiptext',garr[2])
		button.setAttribute('type','menu');
		var newMenupopup = mozileMakeMenu(garr[6]);			// call the menu creation function
		button.appendChild(newMenupopup);
		return button;
	}
	
	var carr = new Array();
	carr = mesCommands[commandIndex];
		
	image = mesImages[0]+carr[3]+mesImages[2];
	
	button = document.createElement('toolbarbutton');
	button.id="mozile-"+carr[0];
	button.setAttribute('class','mozile-toolbar-button');
	button.setAttribute('label',carr[1]);
	button.setAttribute('image',image);
	button.setAttribute('tooltiptext',carr[2]);
	button.setAttribute('active',mozileCheckActive(carr));
	button.setAttribute('oncommand','mozileFormat("'+ carr[0] +'")' ); 
	
	return button;
}
